html {
    background: var(--whoogle-dark-page-bg) !important;
}

body {
    background: var(--whoogle-dark-page-bg) !important;
}

div {
    color: var(--whoogle-dark-text) !important;
}

label {
    color: var(--whoogle-dark-contrast-text) !important;
}

li a {
    color: var(--whoogle-dark-result-url) !important;
}

li {
    color: var(--whoogle-dark-text) !important;
}

.anon-view {
    color: var(--whoogle-dark-text) !important;
    text-decoration: underline;
}

textarea {
    background: var(--whoogle-dark-page-bg) !important;
    color: var(--whoogle-dark-text) !important;
}

a:visited h3 div, a:visited .qXLe6d {
    color: var(--whoogle-dark-result-visited) !important;
}

a:link h3 div, a:link .qXLe6d {
    color: var(--whoogle-dark-result-title) !important;
}

a:link div, a:link .fYyStc {
    color: var(--whoogle-dark-result-url) !important;
}

div span {
    color: var(--whoogle-dark-secondary-text) !important;
}

input {
    background-color: var(--whoogle-dark-page-bg) !important;
    color: var(--whoogle-dark-text) !important;
}

select {
    background: var(--whoogle-dark-page-bg) !important;
    color: var(--whoogle-dark-text) !important;
}

.search-container {
    background-color: var(--whoogle-dark-page-bg) !important;
}

.ZINbbc, .ezO2md {
    overflow: hidden;
    box-shadow: 0 0 0 0 !important;
    background-color: var(--whoogle-dark-result-bg) !important;
    margin-bottom: 10px !important;
    border-radius: 8px !important;
}

.BsXmcf {
    background-color: unset !important;
}

.KP7LCb {
    box-shadow: 0 0 0 0 !important;
}

.BVG0Nb {
    box-shadow: 0 0 0 0 !important;
    background-color: var(--whoogle-dark-page-bg) !important;
}

.ZINbbc.luh4tb {
    background: var(--whoogle-dark-result-bg) !important;
    margin-bottom: 24px !important;
}

.bRsWnc {
    background-color: var(--whoogle-dark-result-bg) !important;
}

.x54gtf {
    background-color: var(--whoogle-dark-divider) !important;
}

.Q0HXG {
    background-color: var(--whoogle-dark-divider) !important;
}

.LKSyXe {
    background-color: var(--whoogle-dark-divider) !important;
}

.home-search {
    border-color: var(--whoogle-dark-element-bg) !important;
}

.sa1toc {
	background: var(--whoogle-dark-page-bg) !important;
}

#search-bar {
    border-color: var(--whoogle-dark-element-bg) !important;
    color: var(--whoogle-dark-text) !important;
    background-color: var(--whoogle-dark-result-bg) !important;
    border-bottom: 2px solid var(--whoogle-dark-element-bg);
}

#search-bar:focus {
    color: var(--whoogle-dark-text) !important;
}

#search-submit {
    border: 1px solid var(--whoogle-dark-element-bg) !important;
    background: var(--whoogle-dark-element-bg) !important;
    color: var(--whoogle-dark-contrast-text) !important;
}

.info-text {
    color: var(--whoogle-dark-contrast-text) !important;
    opacity: 75%;
}

.collapsible {
    color: var(--whoogle-dark-text) !important;
}

.collapsible:after {
    color: var(--whoogle-dark-text) !important;
}

.active {
    background-color: var(--whoogle-dark-element-bg) !important;
    color: var(--whoogle-dark-contrast-text) !important;
}

.content, .result-config {
    background-color: var(--whoogle-dark-element-bg) !important;
    color: var(--whoogle-contrast-text) !important;
}

.active:after {
    color: var(--whoogle-dark-contrast-text) !important;
}

.link {
    color: var(--whoogle-dark-contrast-text);
}

.link-color {
    color: var(--whoogle-dark-result-url) !important;
}

.autocomplete-items {
    border: 1px solid var(--whoogle-dark-element-bg);
}

.autocomplete-items div {
    color: var(--whoogle-dark-text);
    background-color: var(--whoogle-dark-page-bg);
    border-bottom: 1px solid var(--whoogle-dark-element-bg);
}

.autocomplete-items div:hover {
    background-color: var(--whoogle-dark-element-bg);
    color: var(--whoogle-dark-contrast-text) !important;
}

.autocomplete-active {
    background-color: var(--whoogle-dark-element-bg) !important;
    color: var(--whoogle-dark-contrast-text) !important;
}

.footer {
    color: var(--whoogle-dark-text);
}

path {
    fill: var(--whoogle-dark-logo);
}

.header-div {
    background-color: var(--whoogle-dark-result-bg) !important;
}

#search-reset {
    color: var(--whoogle-dark-text) !important;
}

.mobile-search-bar {
    background-color: var(--whoogle-dark-result-bg) !important;
    color: var(--whoogle-dark-text) !important;
}

.search-bar-desktop {
    color: var(--whoogle-dark-text) !important;
}

.ip-text-div, .update_available, .cb_label, .cb {
  color: var(--whoogle-dark-secondary-text) !important;
}

.cb:focus {
  color: var(--whoogle-dark-contrast-text) !important;
}

.desktop-header, .mobile-header {
    background-color: var(--whoogle-dark-result-bg) !important;
}
