/* Colors */
:root {
    /* LIGHT THEME COLORS */
    --whoogle-logo: #685e79;
    --whoogle-page-bg: #ffffff;
    --whoogle-element-bg: #4285f4;
    --whoogle-text: #000000;
    --whoogle-contrast-text: #ffffff;
    --whoogle-secondary-text: #70757a;
    --whoogle-result-bg: #ffffff;
    --whoogle-result-title: #1967d2;
    --whoogle-result-url: #0d652d;
    --whoogle-result-visited: #4b11a8;

    /* DARK THEME COLORS */
    --whoogle-dark-logo: #685e79;
    --whoogle-dark-page-bg: #101020;
    --whoogle-dark-element-bg: #4285f4;
    --whoogle-dark-text: #ffffff;
    --whoogle-dark-contrast-text: #ffffff;
    --whoogle-dark-secondary-text: #bbbbbb;
    --whoogle-dark-result-bg: #212131;
    --whoogle-dark-result-title: #64a7f6;
    --whoogle-dark-result-url: #34a853;
    --whoogle-dark-result-visited: #bbbbff;
}

#whoogle-w {
    fill: #4285f4;
}

#whoogle-h {
    fill: #ea4335;
}

#whoogle-o-1 {
    fill: #fbbc05;
}

#whoogle-o-2 {
    fill: #4285f4;
}

#whoogle-g {
    fill: #34a853;
}

#whoogle-l {
    fill: #ea4335;
}

#whoogle-e {
    fill: #fbbc05;
}
