<footer>
    <p class="footer">
        Whoogle Search v{{ version_number }} ||
        <a class="link" href="https://github.com/benbusby/whoogle-search">{{ translation['github-link'] }}</a>
        {% if has_update %}
             || <span class="update_available">Update Available 🟢</span>
        {% endif %}
    </p>
</footer>
