<form id="search-form" action="search" method="post">
    <input
            type="text"
            name="q"
            style="width: 90%;"
            autofocus="autofocus"
            autocapitalize="none"
            spellcheck="false"
            autocorrect="off"
            placeholder="Whoogle Search"
            autocomplete="off"
            dir="auto">
    <input type="submit" style="width: 9%" id="search-submit" value="Search">
</form>
