#search-bar {
    background: transparent !important;
    padding-right: 50px;
}

#search-reset {
    all: unset;
    margin-left: -50px;
    text-align: center;
    background-color: transparent !important;
    cursor: pointer;
    height: 40px;
    width: 50px;
}
.ZINbbc.xpd.O9g5cc.uUPGi input::-webkit-outer-spin-button,
input::-webkit-inner-spin-button {
  -webkit-appearance: none;
  margin: 0;
}

.cb {
    width: 40%;
    overflow: hidden;
    text-align: left;
    line-height: 28px;
    background: transparent;
    border-radius: 6px;
    border: 1px solid #5f6368;
    font-size: 14px !important;
    height: 36px;
    padding: 0 0 0 12px;
    margin: 10px 10px 10px 0;
}

.conversion_box {
    margin-top: 15px;
}

.ZINbbc.xpd.O9g5cc.uUPGi input:focus-visible {
    outline: 0;
}
